@@ -1,7 +1,7 @@
 import re
 import hashlib
 import sqlparse
-
+from psycopg2 import sql
 
 from velocity.db.core import exceptions
 
@@ -285,6 +285,20 @@ class SQL:
 
     @classmethod
     def update(cls, tx, table, data, where=None, pk=None, excluded=False):
+        """
+        Generate a Postgres UPDATE statement, handling the WHERE clause logic similar
+        to how the SELECT statement does. If you want to do an ON CONFLICT ... DO UPDATE,
+        that logic should generally live in `merge(...)` rather than here.
+
+        :param tx: Database/transaction context object (used by TableHelper)
+        :param table: Table name
+        :param data: Dictionary of columns to update
+        :param where: WHERE clause conditions (dict, list of tuples, or string)
+        :param pk: Primary key dict to merge with `where`
+        :param excluded: If True, creates `col = EXCLUDED.col` expressions (used in upsert)
+        :return: (sql_string, params_tuple)
+        """
+
         if not table:
             raise ValueError("Table name is required.")
         if not pk and not where:
@@ -293,102 +307,110 @@ class SQL:
             raise ValueError("data must be a non-empty mapping of column-value pairs.")
 
         th = TableHelper(tx, table)
-
         set_clauses = []
         vals = []
 
+        # Merge pk into where if pk is provided
         if pk:
             if where:
-                where.update(pk)
+                # If where is a dict, update it; otherwise raise error
+                if isinstance(where, Mapping):
+                    where = dict(where)  # copy to avoid mutation
+                    where.update(pk)
+                else:
+                    raise ValueError(
+                        "Cannot combine 'pk' with a non-dict 'where' clause."
+                    )
             else:
                 where = pk
 
-        # Handle data columns (SET clause)
+        # Build SET clauses
         for col, val in data.items():
-            col = th.resolve_references(
+            col_quoted = th.resolve_references(
                 col, options={"alias_column": False, "alias_table": False}
             )
-
-            # Normal column
             if excluded:
-                set_clauses.append(f"{col} = EXCLUDED.{col}")
+                # For ON CONFLICT DO UPDATE statements, use the EXCLUDED value
+                set_clauses.append(f"{col_quoted} = EXCLUDED.{col_quoted}")
             else:
-                set_clauses.append(f"{col} = %s")
+                set_clauses.append(f"{col_quoted} = %s")
                 vals.append(val)
 
-        # Extract the final where conditions and values
+        # Build WHERE clauses for a normal update (ignored when excluded is True)
         where_clauses = []
         if not excluded:
-            # First handle user-provided WHERE conditions
-            if isinstance(where, Mapping):
-                for key, val in where.items():
-                    col, value = th.make_predicate(key, val)
-                    where_clauses.append(col)
-                    if isinstance(value, tuple):
-                        vals.extend(value)
-                    else:
-                        vals.append(value)
-
-        # Final assembly of SQL
-        sql = []
-        sql.append("UPDATE")
-        if not excluded:
-            if th.foreign_keys:
-                sql.append(
-                    f"{th.quote(table)} AS {th.quote(th.get_table_alias('current_table'))}"
+            if where:
+                if isinstance(where, Mapping):
+                    new_where = []
+                    for key, val in where.items():
+                        new_where.append(th.make_predicate(key, val))
+                    where = new_where
+                if isinstance(where, str):
+                    where_clauses.append(where)
+                else:
+                    for pred, value in where:
+                        where_clauses.append(pred)
+                        if value is None:
+                            pass
+                        elif isinstance(value, tuple):
+                            vals.extend(value)
+                        else:
+                            vals.append(value)
+            if not where_clauses:
+                raise ValueError(
+                    "No WHERE clause could be constructed. Update would affect all rows."
                 )
-            else:
-                sql.append(TableHelper.quote(table))
-        sql.append("SET " + ", ".join(set_clauses))
-        if not excluded:
-            if th.foreign_keys:
-                for key, ref_info in th.foreign_keys.items():
-                    ref_table = ref_info["ref_table"]
-                    sql.append(
-                        f"LEFT JOIN {th.quote(ref_table)} AS {th.quote(ref_info['alias'])} "
-                    )
-                    where_clauses.append(
-                        f"{th.quote(th.get_table_alias('current_table'))}.{th.quote(ref_info['local_column'])} = {th.quote(ref_info['alias'])}.{th.quote(ref_info['ref_column'])}"
-                    )
 
-        if not excluded:
+        # Construct final SQL
+        if excluded:
+            # For an upsert's DO UPDATE, only return the SET clause (no table name, no WHERE)
+            sql_parts = []
+            sql_parts.append("UPDATE")
+            sql_parts.append("SET " + ", ".join(set_clauses))
+            final_sql = sqlparse.format(
+                " ".join(sql_parts), reindent=True, keyword_case="upper"
+            )
+            return final_sql, tuple(vals)
+        else:
+            sql_parts = []
+            sql_parts.append("UPDATE")
+            sql_parts.append(th.quote(table))
+            sql_parts.append("SET " + ", ".join(set_clauses))
             if where_clauses:
-                sql.append("WHERE " + " AND ".join(where_clauses))
-            else:
-                # Without a WHERE, this updates all rows.
-                # If this is not desired, raise an error.
-                if not excluded:
-                    raise ValueError(
-                        "No WHERE clause could be constructed. Update would affect all rows."
-                    )
-
-        # Final assembled query
-        sql = sqlparse.format(" ".join(sql), reindent=True, keyword_case="upper")
-        return sql, tuple(vals)
+                sql_parts.append("WHERE " + " AND ".join(where_clauses))
+            final_sql = sqlparse.format(
+                " ".join(sql_parts), reindent=True, keyword_case="upper"
+            )
+            return final_sql, tuple(vals)
 
     @classmethod
     def insert(cls, table, data):
+        """
+        Generate an INSERT statement.
+        """
+
         keys = []
-        vals = []
+        vals_placeholders = []
         args = []
         for key, val in data.items():
             keys.append(TableHelper.quote(key.lower()))
             if isinstance(val, str) and len(val) > 2 and val[:2] == "@@" and val[2:]:
-                vals.append(val[2:])
+                vals_placeholders.append(val[2:])
             else:
-                vals.append("%s")
+                vals_placeholders.append("%s")
                 args.append(val)
 
-        sql = ["INSERT INTO"]
-        sql.append(TableHelper.quote(table))
-        sql.append("(")
-        sql.append(",".join(keys))
-        sql.append(")")
-        sql.append("VALUES")
-        sql.append("(")
-        sql.append(",".join(vals))
-        sql.append(")")
-        sql = sqlparse.format(" ".join(sql), reindent=True, keyword_case="upper")
+        sql_parts = []
+        sql_parts.append("INSERT INTO")
+        sql_parts.append(TableHelper.quote(table))
+        sql_parts.append("(")
+        sql_parts.append(",".join(keys))
+        sql_parts.append(")")
+        sql_parts.append("VALUES")
+        sql_parts.append("(")
+        sql_parts.append(",".join(vals_placeholders))
+        sql_parts.append(")")
+        sql = sqlparse.format(" ".join(sql_parts), reindent=True, keyword_case="upper")
         return sql, tuple(args)
 
     @classmethod
@@ -397,21 +419,23 @@ class SQL:
             pkeys = tx.table(table).primary_keys()
             if not pkeys:
                 raise ValueError("Primary key required for merge.")
-            # If there are multiple primary keys, we need to use all of them
+            # If there are multiple primary keys, use all of them
             if len(pkeys) > 1:
                 pk = {pk: data[pk] for pk in pkeys}
             else:
                 pk = {pkeys[0]: data[pkeys[0]]}
-            # remove primary key from data
+            # Remove primary keys from data; they will be used in the conflict target
             data = {k: v for k, v in data.items() if k not in pk}
 
+        # Create a merged dictionary for insert (data + primary key columns)
         full_data = {}
         full_data.update(data)
         full_data.update(pk)
 
         sql, vals = cls.insert(table, full_data)
         sql = [sql]
-        vals = list(vals)
+        vals = list(vals)  # Convert to a mutable list
+
         if on_conflict_do_nothing != on_conflict_update:
             sql.append("ON CONFLICT")
             sql.append("(")
@@ -421,15 +445,20 @@ class SQL:
             if on_conflict_do_nothing:
                 sql.append("NOTHING")
             elif on_conflict_update:
-                sql2, vals2 = cls.update(tx, table, data, pk, excluded=True)
-                sql.append(sql2)
-                vals.extend(vals2)
+                # Call update() with excluded=True to produce the SET clause for the upsert.
+                sql_update, vals_update = cls.update(tx, table, data, pk, excluded=True)
+                sql.append(sql_update)
+                # Use list.extend to add the update values to vals.
+                vals.extend(vals_update)
         else:
             raise Exception(
                 "Update on conflict must have one and only one option to complete on conflict."
             )
-        sql = sqlparse.format(" ".join(sql), reindent=True, keyword_case="upper")
-        return sql, tuple(vals)
+
+        import sqlparse
+
+        final_sql = sqlparse.format(" ".join(sql), reindent=True, keyword_case="upper")
+        return final_sql, tuple(vals)
 
     @classmethod
     def version(cls):
@@ -513,9 +542,9 @@ class SQL:
     @classmethod
     def create_table(cls, name, columns={}, drop=False):
         if "." in name:
-            fqtn = name
+            fqtn = TableHelper.quote(name)
         else:
-            fqtn = f"public.{name}"
+            fqtn = f"public.{TableHelper.quote(name)}"
         schema, table = fqtn.split(".")
         name = fqtn.replace(".", "_")
         sql = []
@@ -563,7 +592,7 @@ class SQL:
         )
 
         for key, val in columns.items():
-            key = re.sub("<>!=%", "", key.lower())
+            key = re.sub("<>!=%", "", key)
             if key in system_fields:
                 continue
             sql.append(
@@ -797,7 +826,7 @@ class SQL:
         if "." not in table and schema:
             table = f"{schema}.{table}"
         if isinstance(columns, (list, set)):
-            columns = ",".join([TableHelper.quote(c.lower()) for c in columns])
+            columns = ",".join([TableHelper.quote(c) for c in columns])
         else:
             columns = TableHelper.quote(columns)
         sql = ["CREATE"]
@@ -858,7 +887,7 @@ class SQL:
         if "." not in table and schema:
             table = f"{schema}.{table}"
         if isinstance(columns, (list, set)):
-            columns = ",".join([TableHelper.quote(c.lower()) for c in columns])
+            columns = ",".join([TableHelper.quote(c) for c in columns])
         else:
             columns = TableHelper.quote(columns)
         sql = ["DROP"]
@@ -880,7 +909,7 @@ class SQL:
 
     @classmethod
     def massage_data(cls, data):
-        data = {key.lower(): val for key, val in data.items()}
+        data = {key: val for key, val in data.items()}
         primaryKey = set(cls.GetPrimaryKeyColumnNames())
         if not primaryKey:
             if not cls.Exists():
@@ -895,23 +924,43 @@ class SQL:
 
     @classmethod
     def alter_add(cls, table, columns, null_allowed=True):
+        """
+        Modify the table to add new columns. If the `value` is 'now()', treat it as a
+        TIMESTAMP type (optionally with a DEFAULT now() clause).
+        """
         sql = []
-        null = "NOT NULL" if not null_allowed else ""
+        null_clause = "NOT NULL" if not null_allowed else ""
+
         if isinstance(columns, dict):
-            for key, val in columns.items():
-                key = re.sub("<>!=%", "", key.lower())
-                sql.append(
-                    f"ALTER TABLE {TableHelper.quote(table)} ADD {TableHelper.quote(key)} {TYPES.get_type(val)} {null};"
-                )
-        sql = sqlparse.format(" ".join(sql), reindent=True, keyword_case="upper")
-        return sql, tuple()
+            for col_name, val in columns.items():
+                col_name_clean = re.sub("<>!=%", "", col_name)
+                # If the user wants 'now()' to be recognized as a TIMESTAMP column:
+                if isinstance(val, str) and val.strip().lower() == "@@now()":
+                    # We assume the user wants the type to be TIMESTAMP
+                    # Optionally we can also add `DEFAULT now()` if desired
+                    # so that newly added rows use the current timestamp
+                    col_type = "TIMESTAMP"
+                    sql.append(
+                        f"ALTER TABLE {TableHelper.quote(table)} "
+                        f"ADD {TableHelper.quote(col_name_clean)} {col_type} {null_clause};"
+                    )
+                else:
+                    # Normal code path: rely on your `TYPES.get_type(...)` logic
+                    col_type = TYPES.get_type(val)
+                    sql.append(
+                        f"ALTER TABLE {TableHelper.quote(table)} "
+                        f"ADD {TableHelper.quote(col_name_clean)} {col_type} {null_clause};"
+                    )
+
+        final_sql = sqlparse.format(" ".join(sql), reindent=True, keyword_case="upper")
+        return final_sql, tuple()
 
     @classmethod
     def alter_drop(cls, table, columns):
         sql = [f"ALTER TABLE {TableHelper.quote(table)} DROP COLUMN"]
         if isinstance(columns, dict):
             for key, val in columns.items():
-                key = re.sub("<>!=%", "", key.lower())
+                key = re.sub("<>!=%", "", key)
                 sql.append(f"{key},")
         if sql[-1][-1] == ",":
             sql[-1] = sql[-1][:-1]
@@ -963,7 +1012,7 @@ class SQL:
 
     @classmethod
     def delete(cls, tx, table, where):
-        sql = [f"DELETE FROM {table}"]
+        sql = [f"DELETE FROM {TableHelper.quote(table)}"]
         vals = []
         if where:
             s, v = TableHelper(tx, table).make_where(where)
